@@ -1,8 +1,12 @@
 #!/bin/sh
-srcversion='$MirOS: src/bin/mksh/Build.sh,v 1.313 2008/04/02 16:55:05 tg Exp $'
+srcversion='$MirOS: src/bin/mksh/Build.sh,v 1.349 2008/07/18 11:42:52 tg Exp $'
 #-
 # Environment used: CC CFLAGS CPPFLAGS LDFLAGS LIBS NOWARN NROFF TARGET_OS
 # CPPFLAGS recognised:	MKSH_SMALL MKSH_ASSUME_UTF8 MKSH_NOPWNAM MKSH_NOVI
+#			MKSH_CLS_STRING MKSH_AFREE_DEBUG
+
+LC_ALL=C
+export LC_ALL
 
 v() {
 	$e "$*"
@@ -132,7 +136,7 @@ ac_testn() {
 	fi
 	test ugcc=$phase$ct && $CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN scn.c \
 	    $LIBS 2>&1 | grep 'unrecogni[sz]ed' >/dev/null 2>&1 && fv=$fr
-	rm -f scn.c scn.o $tcfn
+	rm -f scn.c scn.o ${tcfn}*
 	ac_testdone
 }
 
@@ -208,10 +212,10 @@ if test -d mksh || test -d mksh.exe; then
 	echo "$me: Error: ./mksh is a directory!" >&2
 	exit 1
 fi
-rm -f a.exe a.out *core crypt.exp lft mksh mksh.cat1 mksh.exe no *.o \
+rm -f a.exe* a.out* *core crypt.exp lft mksh mksh.cat1 mksh.exe no *.o \
     scn.c signames.inc stdint.h test.sh x
 
-curdir=`pwd` srcdir=`dirname "$0"` check_categories=pdksh
+curdir=`pwd` srcdir=`dirname "$0"` check_categories=
 
 e=echo
 h=1
@@ -259,9 +263,10 @@ warn=
 ccpc=-Wc,
 ccpl=-Wl,
 tsts=
-ccpr='|| rm -f $tcfn'
+ccpr='|| rm -f ${tcfn}*'
 case $TARGET_OS in
 AIX)
+	CPPFLAGS="$CPPFLAGS -D_ALL_SOURCE"
 	if test x"$LDFLAGS" = x""; then
 		LDFLAGS="${ccpl}-bI:crypt.exp"
 		cat >crypt.exp <<-EOF
@@ -274,8 +279,10 @@ AIX)
 	: ${LIBS='-lcrypt'}
 	;;
 BSD/OS)
+	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 CYGWIN*)
+	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 Darwin)
 	;;
@@ -296,8 +303,10 @@ Interix)
 	ccpl='-Y '
 	CPPFLAGS="$CPPFLAGS -D_ALL_SOURCE"
 	: ${LIBS='-lcrypt'}
+	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 IRIX*)
+	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 Linux)
 	CPPFLAGS="$CPPFLAGS -D_GNU_SOURCE"
@@ -306,31 +315,36 @@ Linux)
 MidnightBSD)
 	;;
 Minix)
-	CPPFLAGS="$CPPFLAGS -D_MINIX -D_POSIX_SOURCE"
+	CPPFLAGS="$CPPFLAGS -D_POSIX_SOURCE -D_POSIX_1_SOURCE=2"
 	warn=' and will currently not work'
 #	warn=" but might work with the GNU tools"
 #	warn="$warn${nl}but not with ACK - /usr/bin/cc - yet)"
+	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 MirBSD)
 	;;
 NetBSD)
 	;;
 OpenBSD)
+	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 OSF1)
 	HAVE_SIG_T=0	# incompatible
 	CPPFLAGS="$CPPFLAGS -D_OSF_SOURCE -D_POSIX_C_SOURCE=200112L"
 	CPPFLAGS="$CPPFLAGS -D_XOPEN_SOURCE=600 -D_XOPEN_SOURCE_EXTENDED"
+	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 Plan9)
 	CPPFLAGS="$CPPFLAGS -D_POSIX_SOURCE -D_LIMITS_EXTENSION"
 	CPPFLAGS="$CPPFLAGS -D_BSD_EXTENSION -D_SUSV2_SOURCE"
 	warn=' and will currently not work'
+	CPPFLAGS="$CPPFLAGS -DMKSH_ASSUME_UTF8"
 	;;
 PW32*)
 	HAVE_SIG_T=0	# incompatible
 	warn=' and will currently not work'
 	# missing: killpg() getrlimit()
+	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 SunOS)
 	CPPFLAGS="$CPPFLAGS -D_BSD_SOURCE -D__EXTENSIONS__"
@@ -342,6 +356,7 @@ syllable)
 ULTRIX)
 	: ${CC=cc -YPOSIX}
 	CPPFLAGS="$CPPFLAGS -Dssize_t=int"
+	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 UWIN*)
 	ccpc='-Yc,'
@@ -349,6 +364,7 @@ UWIN*)
 	tsts=" 3<>/dev/tty"
 	warn="; it will compile, but the target"
 	warn="$warn${nl}platform itself is very flakey/unreliable"
+	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 *)
 	warn='; it may or may not work'
@@ -370,7 +386,7 @@ dstversion=`sed -n '/define MKSH_VERSION/s/^.*"\(.*\)".*$/\1/p' $srcdir/sh.h`
 $e "Hi from$ao $bi$srcversion$ao on:"
 case $TARGET_OS in
 Darwin)
-	vv '|' "hwprefs os_type >&2"
+	vv '|' "hwprefs machine_type os_type os_class >&2"
 	vv '|' "uname -a >&2"
 	;;
 IRIX*)
@@ -379,7 +395,7 @@ IRIX*)
 	;;
 OSF1)
 	vv '|' "uname -a >&2"
-	vv '|' "sizer -v >&2"
+	vv '|' "/usr/sbin/sizer -v >&2"
 	;;
 *)
 	vv '|' "uname -a >&2"
@@ -398,6 +414,7 @@ $e $bi$me: Scanning for functions... please ignore any errors.$ao
 # notes:
 # – ICC defines __GNUC__ too
 # – GCC defines __hpux too
+# - LLVM+clang defines __GNUC__ too
 CPP="$CC -E"
 $e ... which compiler seems to be used
 cat >scn.c <<-'EOF'
@@ -435,6 +452,8 @@ cat >scn.c <<-'EOF'
 	ct=tendra
 	#elif defined(__TINYC__)
 	ct=tcc
+	#elif defined(__llvm__) && defined(__clang__)
+	ct=clang
 	#elif defined(__GNUC__)
 	ct=gcc
 	#elif defined(_COMPILER_VERSION)
@@ -471,6 +490,10 @@ bcc)
 	echo >&2 "    produce broken executables. Continue at your own risk,"
 	echo >&2 "    please report success/failure to the developers."
 	;;
+clang)
+	# does not work with current "ccc" compiler driver
+	vv '|' "$CC -version"
+	;;
 dec)
 	vv '|' "$CC -V"
 	;;
@@ -481,11 +504,11 @@ dmc)
 	echo >&2 "    please report success/failure to the developers."
 	;;
 gcc)
-	vv '|' "$CC -v"
+	vv '|' "$CC -v scn.c"
 	vv '|' 'echo `$CC -dumpmachine` gcc`$CC -dumpversion`'
 	;;
 hpcc)
-	vv '|' "$CC -V"
+	vv '|' "$CC -V scn.c"
 	;;
 iar)
 	cat >&2 <<-'EOF'
@@ -546,7 +569,7 @@ sdcc)
 	EOF
 	;;
 sunpro)
-	vv '|' "$CC -v"
+	vv '|' "$CC -V scn.c"
 	;;
 tcc)
 	vv '|' "$CC -v"
@@ -572,7 +595,7 @@ xlc)
 	;;
 esac
 $e "$bi==> which compiler seems to be used...$ao $ui$ct$ao"
-rm -f scn.c scn.o scn a.out a.exe
+rm -f scn.c scn.o scn a.out* a.exe*
 
 case $TARGET_OS in
 HP-UX)
@@ -748,6 +771,8 @@ elif test $ct = tendra; then
 	ac_flags 1 extansi -Xa
 elif test $ct = tcc; then
 	ac_flags 1 boundschk -b
+elif test $ct = clang; then
+	i=1
 fi
 # flags common to a subset of compilers
 if test 1 = $i; then
@@ -760,7 +785,7 @@ phase=x
 NOWARN=$save_NOWARN	# gcc runs with -Werror until here
 ac_test expstmt '' "if the compiler supports statements as expressions" <<-'EOF'
 	#define ksh_isspace(c)	({					\
-		unsigned ksh_isspace_c = (c);				\
+		unsigned int ksh_isspace_c = (c);			\
 		(ksh_isspace_c >= 0x09 && ksh_isspace_c <= 0x0D) ||	\
 		    (ksh_isspace_c == 0x20);				\
 	})
@@ -819,16 +844,6 @@ ac_testn mksh_full '' "if a full-featured mksh is requested" <<-'EOF'
 	#endif
 EOF
 
-ac_testn mksh_defutf8 '' "if to assume UTF-8 is enabled" <<-'EOF'
-	#ifdef MKSH_ASSUME_UTF8
-	/* force a success: we assume UTF-8 by default */
-	int main(void) { return (0); }
-	#else
-	/* force a failure: use setlocale() and nl_langinfo(CODESET) */
-	int main(void) { return (thiswillneverbedefinedIhope()); }
-	#endif
-EOF
-
 if test 0 = $HAVE_MKSH_FULL; then
 	if test $ct = xlc; then
 		ac_flags 1 fnoinline -qnoinline
@@ -836,9 +851,8 @@ if test 0 = $HAVE_MKSH_FULL; then
 		ac_flags 1 fnoinline -fno-inline
 	fi
 
-	: ${HAVE_MKNOD=0} ${HAVE_SETLOCALE_CTYPE=0}
+	: ${HAVE_MKNOD=0}
 	check_categories=$check_categories,smksh
-	test 0 = $HAVE_MKSH_DEFUTF8 || check_categories=$check_categories,dutf
 fi
 
 #
@@ -853,7 +867,7 @@ ac_header libutil.h
 ac_header paths.h
 ac_header stdbool.h
 ac_header grp.h sys/types.h
-ac_header ulimit.h
+ac_header ulimit.h sys/types.h
 ac_header values.h
 
 ac_header '!' stdint.h stdarg.h
@@ -911,13 +925,15 @@ EOF
 ac_testn sig_t <<-'EOF'
 	#include <sys/types.h>
 	#include <signal.h>
-	int main(void) { return ((int)(sig_t)0); }
+	#include <stddef.h>
+	int main(void) { return ((ptrdiff_t)(sig_t)0); }
 EOF
 
 ac_testn sighandler_t '!' sig_t 0 <<-'EOF'
 	#include <sys/types.h>
 	#include <signal.h>
-	int main(void) { return ((int)(sighandler_t)0); }
+	#include <stddef.h>
+	int main(void) { return ((ptrdiff_t)(sighandler_t)0); }
 EOF
 if test 1 = $HAVE_SIGHANDLER_T; then
 	CPPFLAGS="$CPPFLAGS -Dsig_t=sighandler_t"
@@ -927,7 +943,8 @@ fi
 ac_testn __sighandler_t '!' sig_t 0 <<-'EOF'
 	#include <sys/types.h>
 	#include <signal.h>
-	int main(void) { return ((int)(__sighandler_t)0); }
+	#include <stddef.h>
+	int main(void) { return ((ptrdiff_t)(__sighandler_t)0); }
 EOF
 if test 1 = $HAVE___SIGHANDLER_T; then
 	CPPFLAGS="$CPPFLAGS -Dsig_t=__sighandler_t"
@@ -987,13 +1004,17 @@ ac_testn arc4random <<-'EOF'
 EOF
 
 save_LIBS=$LIBS
-if test 0 = $HAVE_ARC4RANDOM && test -f "$srcdir/arc4random.c"; then
-	ac_header sys/sysctl.h
-	addsrcs HAVE_ARC4RANDOM arc4random.c
-	HAVE_ARC4RANDOM=1
-	# ensure isolation of source directory from build directory
-	test -f arc4random.c || cp "$srcdir/arc4random.c" .
-	LIBS="$LIBS arc4random.c"
+if test 0 = $HAVE_ARC4RANDOM; then
+	test -f arc4random.c || if test -f "$srcdir/arc4random.c"; then
+		# ensure isolation of source directory from build directory
+		cp "$srcdir/arc4random.c" .
+	fi
+	if test -f arc4random.c; then
+		ac_header sys/sysctl.h
+		addsrcs HAVE_ARC4RANDOM arc4random.c
+		HAVE_ARC4RANDOM=1
+		LIBS="$LIBS arc4random.c"
+	fi
 fi
 ac_cppflags ARC4RANDOM
 
@@ -1007,7 +1028,7 @@ ac_test arc4random_pushb arc4random 0 <<-'EOF'
 EOF
 LIBS=$save_LIBS
 
-ac_test flock_ex '' 'flock and mmap' <<-'EOF'
+ac_testn flock_ex '' 'flock and mmap' <<-'EOF'
 	#include <sys/types.h>
 	#include <sys/file.h>
 	#include <sys/mman.h>
@@ -1022,7 +1043,7 @@ ac_test mkstemp <<-'EOF'
 	int main(void) { char tmpl[] = "X"; return (mkstemp(tmpl)); }
 EOF
 
-ac_test setlocale_ctype '!' mksh_defutf8 0 'setlocale(LC_CTYPE, "")' <<-'EOF'
+ac_test setlocale_ctype '' 'setlocale(LC_CTYPE, "")' <<-'EOF'
 	#include <locale.h>
 	#include <stddef.h>
 	int main(void) { return ((ptrdiff_t)(void *)setlocale(LC_CTYPE, "")); }
@@ -1044,6 +1065,22 @@ ac_test mknod '' 'if to use mknod(), makedev() and friends' <<-'EOF'
 	}
 EOF
 
+ac_test realpath mksh_full 0 <<-'EOF'
+	#if HAVE_SYS_PARAM_H
+	#include <sys/param.h>
+	#endif
+	#include <stdlib.h>
+	#ifndef PATH_MAX
+	#define PATH_MAX 1024
+	#endif
+	char *res, dst[PATH_MAX];
+	const char src[] = ".";
+	int main(void) {
+		res = realpath(src, dst);
+		return (res == NULL ? 1 : 0);
+	}
+EOF
+
 ac_test revoke mksh_full 0 <<-'EOF'
 	#if HAVE_LIBUTIL_H
 	#include <libutil.h>
@@ -1077,7 +1114,7 @@ ac_test setgroups setresugid 0 <<-'EOF'
 	int main(void) { gid_t gid = 0; return (setgroups(0, &gid)); }
 EOF
 
-ac_test strcasestr setlocale_ctype 1 <<-'EOF'
+ac_test strcasestr <<-'EOF'
 	#include <stddef.h>
 	#include <string.h>
 	int main(int ac, char *av[]) {
@@ -1093,8 +1130,8 @@ EOF
 #
 # check headers for declarations
 #
-save_CC=$CC
-CC="$CC -c -o $tcfn"
+save_CC=$CC; save_LDFLAGS=$LDFLAGS; save_LIBS=$LIBS
+CC="$CC -c -o $tcfn"; LDFLAGS=; LIBS=
 ac_test '!' arc4random_decl arc4random 1 'if arc4random() does not need to be declared' <<-'EOF'
 	#define MKSH_INCLUDES_ONLY
 	#include "sh.h"
@@ -1124,7 +1161,7 @@ ac_test sys_siglist_decl sys_siglist 1 'if sys_siglist[] does not need to be dec
 	#include "sh.h"
 	int main(void) { return (sys_siglist[0][0]); }
 EOF
-CC=$save_CC
+CC=$save_CC; LDFLAGS=$save_LDFLAGS; LIBS=$save_LIBS
 
 #
 # other checks
@@ -1224,18 +1261,23 @@ case $curdir in
 *)	echo "#!$curdir/mksh" >test.sh ;;
 esac
 cat >>test.sh <<-EOF
-	export PATH='$PATH'
+	LC_ALL=C PATH='$PATH'; export LC_ALL PATH
+	test -n "\$KSH_VERSION" || exit 1
 	check_categories=$check_categories
 	print Testing mksh for conformance:
 	fgrep MirOS: '$srcdir/check.t'
 	fgrep MIRBSD '$srcdir/check.t'
 	print "This shell is actually:\\n\\t\$KSH_VERSION"
 	print 'test.sh built for mksh $dstversion'
-	perl=perl5
-	\$perl -e print >/dev/null 2>&1 || perl=perl
-	\$perl -e print >/dev/null 2>&1 || exit 1
-	exec \$perl '$srcdir/check.pl' -s '$srcdir/check.t' \\
-	    -p '$curdir/mksh' -C \$check_categories \$*$tsts
+	cstr='\$os = defined \$^O ? \$^O : "unknown";'
+	cstr="\$cstr"'print \$os . ", Perl version " . \$];'
+	for perli in \$PERL perl5 perl no; do
+		[[ \$perli = no ]] && exit 1
+		perlos=\$(\$perli -e "\$cstr") 2>&- || continue
+		print "Perl interpreter '\$perli' running on '\$perlos'"
+		[[ -n \$perlos ]] && break
+	done
+	exec \$perli '$srcdir/check.pl' -s '$srcdir/check.t' -p '$curdir/mksh' -C \${check_categories#,} \$*$tsts
 EOF
 chmod 755 test.sh
 echo set -x >Rebuild.sh
