@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/eval.c,v 1.45 2008/03/01 22:49:37 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/eval.c,v 1.47 2008/07/12 16:56:38 tg Exp $");
 
 #ifdef MKSH_SMALL
 #define MKSH_NOPWNAM
@@ -297,8 +297,8 @@ expand(const char *cp,	/* input word */
 					if (!st->next) {
 						SubType *newst;
 
-						newst = (SubType *) alloc(
-						    sizeof(SubType), ATEMP);
+						newst = (SubType *)alloc(
+						    sizeof (SubType), ATEMP);
 						newst->next = NULL;
 						newst->prev = st;
 						st->next = newst;
@@ -425,7 +425,8 @@ expand(const char *cp,	/* input word */
 							goto no_repl;
 
 						/* prepare string on which to work */
-						sbeg = s = str_save(str_val(st->var), ATEMP);
+						tpat0 = str_val(st->var);
+						sbeg = s = str_save(tpat0, ATEMP);
 
 						/* first see if we have any match at all */
 						tpat0 = pat;
@@ -589,7 +590,7 @@ expand(const char *cp,	/* input word */
 					 */
 					len = strlen(dp) + 1;
 					setstr(st->var,
-					    debunk((char *) alloc(len, ATEMP),
+					    debunk((char *)alloc(len, ATEMP),
 					    dp, len), KSH_UNWIND_ERROR);
 					x.str = str_val(st->var);
 					type = XSUB;
@@ -769,10 +770,7 @@ expand(const char *cp,	/* input word */
 				    Xlength(ds, dp) == 0) {
 					char *p;
 
-					if ((p = str_nsave(null, 0, ATEMP))
-					    == NULL)
-						internal_errorf("unable "
-						    "to allocate memory");
+					*(p = alloc(1, ATEMP)) = '\0';
 					XPput(*wp, p);
 				}
 				type = XSUBMID;
@@ -1474,7 +1472,7 @@ alt_expand(XPtrV *wp, char *start, char *exp_start, char *end, int fdo)
 				l1 = brace_start - start;
 				l2 = (p - 1) - field_start;
 				l3 = end - brace_end;
-				new = (char *) alloc(l1 + l2 + l3 + 1, ATEMP);
+				new = (char *)alloc(l1 + l2 + l3 + 1, ATEMP);
 				memcpy(new, start, l1);
 				memcpy(new + l1, field_start, l2);
 				memcpy(new + l1 + l2, brace_end, l3);
