@@ -1,9 +1,9 @@
-/*	$OpenBSD: history.c,v 1.35 2006/05/29 18:22:24 otto Exp $	*/
+/*	$OpenBSD: history.c,v 1.36 2008/05/20 00:30:30 fgsch Exp $	*/
 /*	$OpenBSD: trap.c,v 1.22 2005/03/30 17:16:37 deraadt Exp $	*/
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.60 2008/04/02 16:55:06 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.67 2008/07/12 17:16:03 tg Exp $");
 
 /*-
  * MirOS: This is the default mapping type, and need not be specified.
@@ -21,7 +21,7 @@ static int hist_count_lines(unsigned char *, int);
 static int hist_shrink(unsigned char *, int);
 static unsigned char *hist_skip_back(unsigned char *,int *,int);
 static void histload(Source *, unsigned char *, int);
-static void histinsert(Source *, int, unsigned char *);
+static void histinsert(Source *, int, const char *);
 static void writehistfile(int, char *);
 static int sprinkle(int);
 #endif
@@ -68,7 +68,8 @@ c_fc(const char **wp)
 				sflag++;
 			else {
 				size_t len = strlen(p);
-				editor = str_nsave(p, len + 4, ATEMP);
+				editor = alloc(len + 4, ATEMP);
+				memcpy(editor, p, len);
 				memcpy(editor + len, " $_", 4);
 			}
 			break;
@@ -189,9 +190,13 @@ c_fc(const char **wp)
 				    hist_source->line - (int)(histptr - hp));
 			shf_putc('\t', shl_stdout);
 			/* print multi-line commands correctly */
-			for (s = *hp; (t = strchr(s, '\n')); s = t)
-				shf_fprintf(shl_stdout, "%.*s\t",
-				    (int)(++t - s), s);
+			s = *hp;
+			while ((t = strchr(s, '\n'))) {
+				*t = '\0';
+				shf_fprintf(shl_stdout, "%s\n\t", s);
+				*t++ = '\n';
+				s = t;
+			}
 			shf_fprintf(shl_stdout, "%s\n", s);
 		}
 		shf_flush(shl_stdout);
@@ -347,14 +352,14 @@ hist_get(const char *str, int approx, int allow_cur)
 
 	if (getn(str, &n)) {
 		hp = histptr + (n < 0 ? n : (n - hist_source->line));
-		if (hp < history) {
+		if ((ptrdiff_t)hp < (ptrdiff_t)history) {
 			if (approx)
 				hp = hist_get_oldest();
 			else {
 				bi_errorf("%s: not in history", str);
 				hp = NULL;
 			}
-		} else if (hp > histptr) {
+		} else if ((ptrdiff_t)hp > (ptrdiff_t)histptr) {
 			if (approx)
 				hp = hist_get_newest(allow_cur);
 			else {
@@ -550,7 +555,7 @@ init_histvec(void)
 {
 	if (history == (char **)NULL) {
 		histsize = HISTORYSIZE;
-		history = (char **)alloc(histsize*sizeof (char *), APERM);
+		history = (char **)alloc(histsize * sizeof (char *), APERM);
 		histptr = history - 1;
 	}
 }
@@ -635,8 +640,7 @@ hist_init(Source *s)
 	hist_source = s;
 
 #if HAVE_PERSISTENT_HISTORY
-	hname = str_val(global("HISTFILE"));
-	if (hname == NULL)
+	if ((hname = str_val(global("HISTFILE"))) == NULL)
 		return;
 	hname = str_save(hname, APERM);
 
@@ -673,17 +677,27 @@ hist_init(Source *s)
 			if (base != (unsigned char *)MAP_FAILED)
 				munmap((caddr_t)base, hsize);
 			hist_finish();
-			unlink(hname);
+			if (unlink(hname) /* fails */)
+				goto hiniterr;
 			goto retry;
 		}
 		if (hsize > 2) {
+			int rv = 0;
+
 			lines = hist_count_lines(base+2, hsize-2);
 			if (lines > histsize) {
 				/* we need to make the file smaller */
 				if (hist_shrink(base, hsize))
-					unlink(hname);
+					rv = unlink(hname);
 				munmap((caddr_t)base, hsize);
 				hist_finish();
+				if (rv) {
+ hiniterr:
+					bi_errorf("cannot unlink HISTFILE %s"
+					    " - %s", hname, strerror(errno));
+					hsize = 0;
+					return;
+				}
 				goto retry;
 			}
 		}
@@ -841,7 +855,7 @@ histload(Source *s, unsigned char *base, int bytes)
 				/* worry about line numbers */
 				if (histptr >= history && lno-1 != s->line) {
 					/* a replacement ? */
-					histinsert(s, lno, line);
+					histinsert(s, lno, (char *)line);
 				} else {
 					s->line = lno;
 					histsave(lno, (char *)line, 0);
@@ -856,15 +870,15 @@ histload(Source *s, unsigned char *base, int bytes)
  *	Insert a line into the history at a specified number
  */
 static void
-histinsert(Source *s, int lno, unsigned char *line)
+histinsert(Source *s, int lno, const char *line)
 {
 	char **hp;
 
-	if (lno >= s->line-(histptr-history) && lno <= s->line) {
-		hp = &histptr[lno-s->line];
+	if (lno >= s->line - (histptr - history) && lno <= s->line) {
+		hp = &histptr[lno - s->line];
 		if (*hp)
 			afree((void*)*hp, APERM);
-		*hp = str_save((char *)line, APERM);
+		*hp = str_save(line, APERM);
 	}
 }
 
@@ -991,9 +1005,9 @@ inittraps(void)
 			else {
 				char *s;
 
-				sigtraps[i].name = s = str_save(
-				    !strncasecmp(cs, "SIG", 3) ? cs + 3 : cs,
-				    APERM);
+				if (!strncasecmp(cs, "SIG", 3))
+					cs += 3;
+				sigtraps[i].name = s = str_save(cs, APERM);
 				while ((*s = ksh_toupper(*s)))
 					++s;
 			}
