@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/jobs.c,v 1.33 2008/04/01 21:50:58 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/jobs.c,v 1.36 2008/05/17 19:03:25 tg Exp $");
 
 /* Order important! */
 #define PRUNNING	0
@@ -99,9 +99,8 @@ static int32_t njobs;		/* # of jobs started */
 static volatile sig_atomic_t held_sigchld;
 
 static struct shf	*shl_j;
-static int		ttypgrp_ok;	/* set if can use tty pgrps */
+static bool		ttypgrp_ok;	/* set if can use tty pgrps */
 static pid_t		restore_ttypgrp = -1;
-static pid_t		our_pgrp;
 static int const	tt_sigs[] = { SIGTSTP, SIGTTIN, SIGTTOU };
 
 static void		j_set_async(Job *);
@@ -221,14 +220,7 @@ j_change(void)
 			tty_init(false);
 
 		/* no controlling tty, no SIGT* */
-		ttypgrp_ok = use_tty && tty_fd >= 0 && tty_devtty;
-
-		if (ttypgrp_ok && (our_pgrp = getpgrp()) < 0) {
-			warningf(false, "j_init: getpgrp() failed: %s",
-			    strerror(errno));
-			ttypgrp_ok = 0;
-		}
-		if (ttypgrp_ok) {
+		if ((ttypgrp_ok = use_tty && tty_fd >= 0 && tty_devtty)) {
 			setsig(&sigtraps[SIGTTIN], SIG_DFL,
 			    SS_RESTORE_ORIG|SS_FORCE);
 			/* wait to be given tty (POSIX.1, B.2, job control) */
@@ -239,10 +231,10 @@ j_change(void)
 					warningf(false,
 					    "j_init: tcgetpgrp() failed: %s",
 					    strerror(errno));
-					ttypgrp_ok = 0;
+					ttypgrp_ok = false;
 					break;
 				}
-				if (ttypgrp == our_pgrp)
+				if (ttypgrp == kshpgrp)
 					break;
 				kill(0, SIGTTIN);
 			}
@@ -250,21 +242,21 @@ j_change(void)
 		for (i = NELEM(tt_sigs); --i >= 0; )
 			setsig(&sigtraps[tt_sigs[i]], SIG_IGN,
 			    SS_RESTORE_DFL|SS_FORCE);
-		if (ttypgrp_ok && our_pgrp != kshpid) {
+		if (ttypgrp_ok && kshpgrp != kshpid) {
 			if (setpgid(0, kshpid) < 0) {
 				warningf(false,
 				    "j_init: setpgid() failed: %s",
 				    strerror(errno));
-				ttypgrp_ok = 0;
+				ttypgrp_ok = false;
 			} else {
 				if (tcsetpgrp(tty_fd, kshpid) < 0) {
 					warningf(false,
 					    "j_init: tcsetpgrp() failed: %s",
 					    strerror(errno));
-					ttypgrp_ok = 0;
+					ttypgrp_ok = false;
 				} else
-					restore_ttypgrp = our_pgrp;
-				our_pgrp = kshpid;
+					restore_ttypgrp = kshpgrp;
+				kshpgrp = kshpid;
 			}
 		}
 		if (use_tty && !ttypgrp_ok)
@@ -272,7 +264,7 @@ j_change(void)
 		if (tty_fd >= 0)
 			tcgetattr(tty_fd, &tty_state);
 	} else {
-		ttypgrp_ok = 0;
+		ttypgrp_ok = false;
 		if (Flag(FTALKING))
 			for (i = NELEM(tt_sigs); --i >= 0; )
 				setsig(&sigtraps[tt_sigs[i]], SIG_IGN,
@@ -426,7 +418,7 @@ exchild(struct op *t, int flags, /* used if XPCLOSE or XCCLOSE */ int close_fd)
 		}
 		remove_job(j, "child");	/* in case of $(jobs) command */
 		nzombie = 0;
-		ttypgrp_ok = 0;
+		ttypgrp_ok = false;
 		Flag(FMONITOR) = 0;
 		Flag(FTALKING) = 0;
 		tty_close();
@@ -682,12 +674,10 @@ j_resume(const char *cp, int bg)
 			j->flags &= ~JF_FG;
 			if (ttypgrp_ok && (j->flags & JF_SAVEDTTY))
 				tcsetattr(tty_fd, TCSADRAIN, &tty_state);
-			if (ttypgrp_ok && tcsetpgrp(tty_fd, our_pgrp) < 0) {
+			if (ttypgrp_ok && tcsetpgrp(tty_fd, kshpgrp) < 0)
 				warningf(true,
-				    "fg: 2nd tcsetpgrp(%d, %d) failed: %s",
-				    tty_fd, (int) our_pgrp,
-				    strerror(errno));
-			}
+				    "fg: 2nd tcsetpgrp(%d, %ld) failed: %s",
+				    tty_fd, (long)kshpgrp, strerror(errno));
 		}
 		sigprocmask(SIG_SETMASK, &omask, NULL);
 		bi_errorf("cannot continue job %s: %s",
@@ -948,12 +938,10 @@ j_waitj(Job *j,
 			if (j->state == PSTOPPED &&
 			    (j->saved_ttypgrp = tcgetpgrp(tty_fd)) >= 0)
 				j->flags |= JF_SAVEDTTYPGRP;
-			if (tcsetpgrp(tty_fd, our_pgrp) < 0) {
+			if (tcsetpgrp(tty_fd, kshpgrp) < 0)
 				warningf(true,
-				    "j_waitj: tcsetpgrp(%d, %d) failed: %s",
-				    tty_fd, (int) our_pgrp,
-					strerror(errno));
-			}
+				    "j_waitj: tcsetpgrp(%d, %ld) failed: %s",
+				    tty_fd, (long)kshpgrp, strerror(errno));
 			if (j->state == PSTOPPED) {
 				j->flags |= JF_SAVEDTTY;
 				tcgetattr(tty_fd, &j->ttystate);
@@ -1408,7 +1396,7 @@ new_job(void)
 		newj = free_jobs;
 		free_jobs = free_jobs->next;
 	} else
-		newj = (Job *) alloc(sizeof(Job), APERM);
+		newj = (Job *)alloc(sizeof (Job), APERM);
 
 	/* brute force method */
 	for (i = 1; ; i++) {
@@ -1435,7 +1423,7 @@ new_proc(void)
 		p = free_procs;
 		free_procs = free_procs->next;
 	} else
-		p = (Proc *) alloc(sizeof(Proc), APERM);
+		p = (Proc *)alloc(sizeof (Proc), APERM);
 
 	return p;
 }
