@@ -1,8 +1,8 @@
-/*	$OpenBSD: lex.c,v 1.43 2007/06/02 16:40:59 moritz Exp $	*/
+/*	$OpenBSD: lex.c,v 1.44 2008/07/03 17:52:08 otto Exp $	*/
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/lex.c,v 1.57 2008/03/28 13:46:53 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/lex.c,v 1.66 2008/07/12 16:56:39 tg Exp $");
 
 /*
  * states while lexing word
@@ -192,6 +192,10 @@ yylex(int cf)
 		c = getsc();
 		if (c == '<') {
 			state = SHERESTRING;
+			while ((c = getsc()) == ' ' || c == '\t')
+				;
+			ungetsc(c);
+			c = '<';
 			goto accept_nonword;
 		}
 		ungetsc(c);
@@ -771,20 +775,37 @@ yylex(int cf)
 		state = SBASE;
 
 	dp = Xstring(ws, wp);
-	if ((c == '<' || c == '>') && state == SBASE &&
-	    ((c2 = Xlength(ws, wp)) == 0 ||
-	    (c2 == 2 && dp[0] == CHAR && ksh_isdigit(dp[1])))) {
-		struct ioword *iop = (struct ioword *) alloc(sizeof(*iop), ATEMP);
+	if ((c == '<' || c == '>' || c == '&') && state == SBASE) {
+		struct ioword *iop = (struct ioword *)alloc(sizeof (*iop),
+		    ATEMP);
 
-		if (c2 == 2)
-			iop->unit = dp[1] - '0';
-		else
-			iop->unit = c == '>'; /* 0 for <, 1 for > */
+		if (Xlength(ws, wp) == 0)
+			iop->unit = c == '<' ? 0 : 1;
+		else for (iop->unit = 0, c2 = 0; c2 < Xlength(ws, wp); c2 += 2) {
+			if (dp[c2] != CHAR)
+				goto no_iop;
+			if (!ksh_isdigit(dp[c2 + 1]))
+				goto no_iop;
+			iop->unit = (iop->unit * 10) + dp[c2 + 1] - '0';
+		}
+
+		if (iop->unit >= FDBASE)
+			goto no_iop;
+
+		if (c == '&') {
+			if ((c2 = getsc()) != '>') {
+				ungetsc(c2);
+				goto no_iop;
+			}
+			c = c2;
+			iop->flag = IOBASH;
+		} else
+			iop->flag = 0;
 
 		c2 = getsc();
 		/* <<, >>, <> are ok, >< is not */
 		if (c == c2 || (c == '<' && c2 == '>')) {
-			iop->flag = c == c2 ?
+			iop->flag |= c == c2 ?
 			    (c == '>' ? IOCAT : IOHERE) : IORDWR;
 			if (iop->flag == IOHERE) {
 				if ((c2 = getsc()) == '-')
@@ -793,9 +814,9 @@ yylex(int cf)
 					ungetsc(c2);
 			}
 		} else if (c2 == '&')
-			iop->flag = IODUP | (c == '<' ? IORDUP : 0);
+			iop->flag |= IODUP | (c == '<' ? IORDUP : 0);
 		else {
-			iop->flag = c == '>' ? IOWRITE : IOREAD;
+			iop->flag |= c == '>' ? IOWRITE : IOREAD;
 			if (c == '>' && c2 == '|')
 				iop->flag |= IOCLOB;
 			else
@@ -808,6 +829,8 @@ yylex(int cf)
 		Xfree(ws, wp);	/* free word */
 		yylval.iop = iop;
 		return REDIR;
+ no_iop:
+		;
 	}
 
 	if (wp == dp && state == SBASE) {
@@ -1012,7 +1035,7 @@ pushs(int type, Area *areap)
 {
 	Source *s;
 
-	s = (Source *)alloc(sizeof(Source), areap);
+	s = (Source *)alloc(sizeof (Source), areap);
 	s->type = type;
 	s->str = null;
 	s->start = NULL;
@@ -1134,9 +1157,7 @@ getsc__(void)
 		    (((const unsigned char *)(s->str))[0] == 0xBB) &&
 		    (((const unsigned char *)(s->str))[1] == 0xBF)) {
 			s->str += 2;
-#if !defined(MKSH_ASSUME_UTF8) || !defined(MKSH_SMALL)
 			Flag(FUTFHACK) = 1;
-#endif
 			goto getsc_again;
 		}
 	}
@@ -1269,9 +1290,10 @@ set_prompt(int to, Source *s)
 				 * unwinding its stack through this code as it
 				 * exits.
 				 */
-			} else
-				prompt = str_save(substitute(ps1, 0),
-				    saved_atemp);
+			} else {
+				char *cp = substitute(ps1, 0);
+				prompt = str_save(cp, saved_atemp);
+			}
 			quitenv(NULL);
 		}
 		break;
@@ -1310,7 +1332,7 @@ dopprompt(const char *cp, int ntruncate, int doprint)
 				columns--;
 		} else if (*cp == delimiter)
 			indelimit = !indelimit;
-		else if (Flag(FUTFHACK) && ((unsigned)*cp > 0x7F)) {
+		else if (Flag(FUTFHACK) && ((unsigned char)*cp > 0x7F)) {
 			const char *cp2;
 			columns += utf_widthadj(cp, &cp2);
 			if (doprint && (indelimit ||
@@ -1503,7 +1525,7 @@ getsc_bn(void)
 static Lex_state *
 push_state_(State_info *si, Lex_state *old_end)
 {
-	Lex_state	*new = alloc(sizeof(Lex_state) * STATE_BSIZE, ATEMP);
+	Lex_state *new = alloc(sizeof (Lex_state) * STATE_BSIZE, ATEMP);
 
 	new[0].ls_info.base = old_end;
 	si->base = &new[0];
