@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.43 2007/05/31 20:47:44 otto Exp $	*/
+/*	$OpenBSD: main.c,v 1.44 2008/07/05 07:25:18 djm Exp $	*/
 /*	$OpenBSD: tty.c,v 1.9 2006/03/14 22:08:01 deraadt Exp $	*/
 /*	$OpenBSD: io.c,v 1.22 2006/03/17 16:30:13 millert Exp $	*/
 /*	$OpenBSD: table.c,v 1.12 2005/12/11 20:31:21 otto Exp $	*/
@@ -13,7 +13,7 @@
 #include <locale.h>
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/main.c,v 1.95 2008/04/01 20:40:21 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/main.c,v 1.101 2008/07/10 18:34:08 tg Exp $");
 
 extern char **environ;
 
@@ -32,8 +32,8 @@ static const char initsubs[] = "${PS2=> } ${PS3=#? } ${PS4=+ }";
 static const char *initcoms[] = {
 	"typeset", "-r", initvsn, NULL,
 	"typeset", "-x", "SHELL", "PATH", "HOME", NULL,
-	"typeset", "-i", "PPID", "OPTIND=1", NULL,
-	"eval", "typeset -i RANDOM SECONDS=\"${SECONDS-0}\" TMOUT=\"${TMOUT-0}\"", NULL,
+	"typeset", "-i10", "OPTIND=1", "PGRP", "PPID", "USER_ID", NULL,
+	"eval", "typeset -i10 RANDOM SECONDS=\"${SECONDS-0}\" TMOUT=\"${TMOUT-0}\"", NULL,
 	"alias", "integer=typeset -i", "local=typeset", NULL,
 	"alias",
 	"hash=alias -t",	/* not "alias -t --": hash -r needs to work */
@@ -213,7 +213,6 @@ main(int argc, const char *argv[])
 #if HAVE_ARC4RANDOM
 	Flag(FARC4RANDOM) = 2;	/* use arc4random(3) until $RANDOM is written */
 #endif
-	setint(global("PPID"), (long)ppid);
 
 	for (wp = initcoms; *wp != NULL; wp++) {
 		shcomexec(wp);
@@ -228,6 +227,9 @@ main(int argc, const char *argv[])
 	    (!ksheuid && !strchr(str_val(vp), '#')))
 		/* setstr can't fail here */
 		setstr(vp, safe_prompt, KSH_RETURN_ERROR);
+	setint(global("PGRP"), (long)(kshpgrp = getpgrp()));
+	setint(global("PPID"), (long)ppid);
+	setint(global("USER_ID"), (long)ksheuid);
 
 	/* Set this before parsing arguments */
 #if HAVE_SETRESUGID
@@ -285,22 +287,29 @@ main(int argc, const char *argv[])
 	/* Do this after j_init(), as tty_fd is not initialised 'til then */
 	if (Flag(FTALKING)) {
 #ifndef MKSH_ASSUME_UTF8
-#if HAVE_SETLOCALE_CTYPE
 #define isuc(x)	(((x) != NULL) && \
 		    (stristr((x), "UTF-8") || stristr((x), "utf8")))
 		/* Check if we're in a UTF-8 locale */
 		if (!Flag(FUTFHACK)) {
 			const char *ccp;
 
+#if HAVE_SETLOCALE_CTYPE
 			ccp = setlocale(LC_CTYPE, "");
 #if HAVE_LANGINFO_CODESET
 			if (!isuc(ccp))
 				ccp = nl_langinfo(CODESET);
+#endif
+#else
+			ccp = getenv("LC_ALL");
+			if (!ccp || !*ccp) {
+				ccp = getenv("LC_CTYPE");
+				if (!ccp || !*ccp)
+					ccp = getenv("LANG");
+			}
 #endif
 			Flag(FUTFHACK) = isuc(ccp);
 		}
 #undef isuc
-#endif
 #else
 		Flag(FUTFHACK) = 1;
 #endif
@@ -583,7 +592,7 @@ newenv(int type)
 {
 	struct env *ep;
 
-	ep = (struct env *) alloc(sizeof(*ep), ATEMP);
+	ep = (struct env *)alloc(sizeof (*ep), ATEMP);
 	ep->type = type;
 	ep->flags = 0;
 	ainit(&ep->area);
@@ -629,7 +638,7 @@ quitenv(struct shf *shf)
 				 * dump a core..
 				 */
 				if ((sig == SIGINT || sig == SIGTERM) &&
-				    getpgrp() == kshpid) {
+				    (kshpgrp == kshpid)) {
 					setsig(&sigtraps[sig], SIG_DFL,
 					    SS_RESTORE_CURR | SS_FORCE);
 					kill(0, sig);
@@ -998,32 +1007,35 @@ check_fd(const char *name, int mode, const char **emsgp)
 {
 	int fd, fl;
 
-	if (ksh_isdigit(name[0]) && !name[1]) {
-		if ((fl = fcntl(fd = name[0] - '0', F_GETFL, 0)) < 0) {
-			if (emsgp)
-				*emsgp = "bad file descriptor";
-			return -1;
-		}
-		fl &= O_ACCMODE;
-		/* X_OK is a kludge to disable this check for dups (x<&1):
-		 * historical shells never did this check (XXX don't know what
-		 * posix has to say).
-		 */
-		if (!(mode & X_OK) && fl != O_RDWR &&
-		    (((mode & R_OK) && fl != O_RDONLY) ||
-		    ((mode & W_OK) && fl != O_WRONLY))) {
-			if (emsgp)
-				*emsgp = (fl == O_WRONLY) ?
-				    "fd not open for reading" :
-				    "fd not open for writing";
-			return -1;
-		}
-		return fd;
-	} else if (name[0] == 'p' && !name[1])
-		return coproc_getfd(mode, emsgp);
-	if (emsgp)
-		*emsgp = "illegal file descriptor name";
-	return -1;
+	if (name[0] == 'p' && !name[1])
+		return (coproc_getfd(mode, emsgp));
+	for (fd = 0; ksh_isdigit(*name); ++name)
+		fd = (fd * 10) + *name - '0';
+	if (*name || fd >= FDBASE) {
+		if (emsgp)
+			*emsgp = "illegal file descriptor name";
+		return (-1);
+	}
+	if ((fl = fcntl(fd, F_GETFL, 0)) < 0) {
+		if (emsgp)
+			*emsgp = "bad file descriptor";
+		return (-1);
+	}
+	fl &= O_ACCMODE;
+	/* X_OK is a kludge to disable this check for dups (x<&1):
+	 * historical shells never did this check (XXX don't know what
+	 * posix has to say).
+	 */
+	if (!(mode & X_OK) && fl != O_RDWR && (
+	    ((mode & R_OK) && fl != O_RDONLY) ||
+	    ((mode & W_OK) && fl != O_WRONLY))) {
+		if (emsgp)
+			*emsgp = (fl == O_WRONLY) ?
+			    "fd not open for reading" :
+			    "fd not open for writing";
+		return (-1);
+	}
+	return (fd);
 }
 
 /* Called once from main */
@@ -1124,7 +1136,7 @@ maketemp(Area *ap, Temp_type type, struct temp **tlist)
 	pathname = tempnam(dir, "mksh.");
 	len = ((pathname == NULL) ? 0 : strlen(pathname)) + 1;
 #endif
-	tp = (struct temp *) alloc(sizeof(struct temp) + len, ap);
+	tp = (struct temp *)alloc(sizeof (struct temp) + len, ap);
 	tp->name = (char *)&tp[1];
 #if !HAVE_MKSTEMP
 	if (pathname == NULL)
