@@ -1,4 +1,4 @@
-/*	$OpenBSD: misc.c,v 1.32 2007/08/02 11:05:54 fgsch Exp $	*/
+/*	$OpenBSD: misc.c,v 1.34 2008/07/12 12:33:42 miod Exp $	*/
 /*	$OpenBSD: path.c,v 1.12 2005/03/30 17:16:37 deraadt Exp $	*/
 
 #include "sh.h"
@@ -6,8 +6,7 @@
 #include <grp.h>
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/misc.c,v 1.72 2008/04/11 19:55:23 tg Exp $\t"
-	MKSH_SH_H_ID);
+__RCSID("$MirOS: src/bin/mksh/misc.c,v 1.85 2008/07/18 11:33:12 tg Exp $");
 
 #undef USE_CHVT
 #if defined(TIOCSCTTY) && !defined(MKSH_SMALL)
@@ -35,7 +34,7 @@ static char *do_phys_path(XString *, char *, const char *);
 void
 setctypes(const char *s, int t)
 {
-	unsigned i;
+	unsigned int i;
 
 	if (t & C_IFS) {
 		for (i = 0; i < UCHAR_MAX + 1; i++)
@@ -64,31 +63,37 @@ initctypes(void)
 	setctypes(" \n\t\"#$&'()*;<>?[]\\`|", C_QUOTE);
 }
 
-/* Allocate a string of size n+1 and copy upto n characters from the possibly
- * NUL terminated string s into it.  Always returns a NUL terminated string
- * (unless n < 0).
- */
+#if defined(MKSH_SMALL) || !HAVE_EXPSTMT
 char *
 str_nsave(const char *s, int n, Area *ap)
 {
-	char *ns = NULL;
+	char *rv = NULL;
 
-	if (n >= 0 && s)
-		strlcpy(ns = alloc(n + 1, ap), s, n + 1);
-	return (ns);
+	if ((n >= 0) && (s != NULL))
+		strlcpy(rv = alloc(n + 1, ap), s, n + 1);
+	return (rv);
 }
 
-#ifdef MKSH_SMALL
 char *
 str_save(const char *s, Area *ap)
 {
-	return (str_nsave(s, s ? strlen(s) : 0, ap));
+#ifdef MKSH_SMALL
+	return (s ? str_nsave(s, strlen(s), ap) : NULL);
+#else
+	char *rv = NULL;
+
+	if (s != NULL) {
+		size_t sz = strlen(s) + 1;
+		strlcpy(rv = alloc(sz, ap), s, sz);
+	}
+	return (rv);
+#endif
 }
 #endif
 
 /* called from XcheckN() to grow buffer */
 char *
-Xcheck_grow_(XString *xsp, const char *xp, unsigned more)
+Xcheck_grow_(XString *xsp, const char *xp, unsigned int more)
 {
 	const char *old_beg = xsp->beg;
 
@@ -185,7 +190,7 @@ options_fmt_entry(const void *arg, int i, char *buf, int buflen)
 static void
 printoptions(int verbose)
 {
-	unsigned i;
+	unsigned int i;
 
 	if (verbose) {
 		struct options_info oi;
@@ -216,14 +221,14 @@ printoptions(int verbose)
 char *
 getoptions(void)
 {
-	unsigned i;
+	unsigned int i;
 	char m[(int) FNFLAGS + 1];
 	char *cp = m;
 
 	for (i = 0; i < NELEM(options); i++)
 		if (options[i].c && Flag(i))
 			*cp++ = options[i].c;
-	return (str_nsave(m, cp - m, ATEMP));
+	return (str_nsave_(m, cp - m, ATEMP));
 }
 
 /* change a Flag(*) value; takes care of special actions */
@@ -514,7 +519,7 @@ gmatchx(const char *s, const char *p, bool isfile)
 		size_t len = pe - p + 1;
 		char tbuf[64];
 		char *t = len <= sizeof(tbuf) ? tbuf :
-		    (char *) alloc(len, ATEMP);
+		    (char *)alloc(len, ATEMP);
 		debunk(t, p, len);
 		return !strcmp(t, s);
 	}
@@ -954,11 +959,8 @@ print_columns(struct shf *shf, int n,
     char *(*func) (const void *, int, char *, int),
     const void *arg, int max_width, int prefcol)
 {
-	char *str = (char *) alloc(max_width + 1, ATEMP);
-	int i;
-	int r, c;
-	int rows, cols;
-	int nspace;
+	char *str = (char *)alloc(max_width + 1, ATEMP);
+	int i, r, c, rows, cols, nspace;
 
 	/* max_width + 1 for the space.  Note that no space
 	 * is printed after the last column to avoid problems
@@ -1424,6 +1426,7 @@ strstr(char *b, const char *l)
 }
 #endif
 
+#ifndef MKSH_ASSUME_UTF8
 #if !HAVE_STRCASESTR
 const char *
 stristr(const char *b, const char *l)
@@ -1443,10 +1446,11 @@ stristr(const char *b, const char *l)
 	return (b - 1);
 }
 #endif
+#endif
 
 #if !HAVE_EXPSTMT
 bool
-ksh_isspace_(unsigned ksh_isspace_c)
+ksh_isspace_(unsigned int ksh_isspace_c)
 {
 	return ((ksh_isspace_c >= 0x09 && ksh_isspace_c <= 0x0D) ||
 	    (ksh_isspace_c == 0x20));
