@@ -8,8 +8,9 @@
 /*	$OpenBSD: c_test.h,v 1.4 2004/12/20 11:34:26 otto Exp $	*/
 /*	$OpenBSD: tty.h,v 1.5 2004/12/20 11:34:26 otto Exp $	*/
 
-#define MKSH_SH_H_ID "$MirOS: src/bin/mksh/sh.h,v 1.204 2008/04/11 19:55:24 tg Exp $"
-#define MKSH_VERSION "R33 2008/04/11"
+#ifdef __dietlibc__
+#define _BSD_SOURCE	/* live, BSD, live! */
+#endif
 
 #if HAVE_SYS_PARAM_H
 #include <sys/param.h>
@@ -98,6 +99,11 @@
 #define __RCSID(x)	__IDSTRING(rcsid,x)
 #define __SCCSID(x)	__IDSTRING(sccsid,x)
 
+#ifdef EXTERN
+__RCSID("$MirOS: src/bin/mksh/sh.h,v 1.229 2008/07/18 11:33:12 tg Exp $");
+#endif
+#define MKSH_VERSION "R35 2008/07/18"
+
 #ifndef MKSH_INCLUDES_ONLY
 
 /* extra types */
@@ -156,14 +162,17 @@ typedef int bool;
 #if HAVE_EXPSTMT
 /* this macro must not evaluate its arguments several times */
 #define ksh_isspace(c)	({					\
-	unsigned ksh_isspace_c = (c);				\
+	unsigned int ksh_isspace_c = (c);			\
 	(ksh_isspace_c >= 0x09 && ksh_isspace_c <= 0x0D) ||	\
 	    (ksh_isspace_c == 0x20);				\
 })
 #else
-#define ksh_isspace(c)	ksh_isspace_((unsigned)(c))
+#define ksh_isspace(c)	ksh_isspace_((unsigned int)(c))
 #endif
 
+#ifndef PATH_MAX
+#define PATH_MAX	1024
+#endif
 #ifndef S_ISLNK
 #define S_ISLNK(m)	((m & 0170000) == 0120000)
 #endif
@@ -259,8 +268,13 @@ extern int __cdecl setegid(gid_t);
 /* Table flag type - needs > 16 and < 32 bits */
 typedef int32_t Tflag;
 
+#ifdef MKSH_SMALL
 #define NUFILE		32	/* Number of user-accessible files */
 #define FDBASE		10	/* First file usable by Shell */
+#else
+#define NUFILE		56	/* Number of user-accessible files */
+#define FDBASE		24	/* First file usable by Shell */
+#endif
 
 /* Make MAGIC a char that might be printed to make bugs more obvious, but
  * not a char that is used often. Also, can't use the high bit as it causes
@@ -278,6 +292,7 @@ typedef int32_t Tflag;
 EXTERN const char *kshname;	/* $0 */
 EXTERN pid_t kshpid;		/* $$, shell pid */
 EXTERN pid_t procpid;		/* pid of executing process */
+EXTERN pid_t kshpgrp;		/* process group of shell */
 EXTERN uid_t ksheuid;		/* effective uid of shell */
 EXTERN int exstat;		/* exit status */
 EXTERN int subst_exstat;	/* exit status of last $(..)/`..` */
@@ -960,6 +975,7 @@ struct ioword {
 #define IOCLOB	BIT(6)	/* >|, override -o noclobber */
 #define IORDUP	BIT(7)	/* x<&y (as opposed to x>&y) */
 #define IONAMEXP BIT(8)	/* name has been expanded */
+#define IOBASH	BIT(9)	/* &> etc. */
 
 /* execute/exchild flags */
 #define XEXEC	BIT(0)		/* execute without forking */
@@ -1054,7 +1070,7 @@ typedef char *XStringP;
 #define Xsavepos(xs, xp) ((xp) - (xs).beg)
 #define Xrestpos(xs, xp, n) ((xs).beg + (n))
 
-char *Xcheck_grow_(XString *, const char *, unsigned);
+char *Xcheck_grow_(XString *, const char *, unsigned int);
 
 /*
  * expandable vector of generic pointers
@@ -1067,7 +1083,7 @@ typedef struct XPtrV {
 
 #define XPinit(x, n) do {					\
 	void **vp__;						\
-	vp__ = (void**) alloc(sizeofN(void*, (n)), ATEMP);	\
+	vp__ = (void**)alloc(sizeofN(void*, (n)), ATEMP);	\
 	(x).cur = (x).beg = vp__;				\
 	(x).end = vp__ + (n);					\
 } while (0)
@@ -1203,23 +1219,21 @@ EXTERN struct timeval j_usrtime, j_systime;
 /* alloc.c */
 Area *ainit(Area *);
 void afreeall(Area *);
-void *alloc(size_t, Area *);
+void *alloc(size_t, Area *);	/* cannot fail */
 void *aresize(void *, size_t, Area *);
-void afree(void *, Area *);
-#define afreechk(s)	do {		\
-	if (s)				\
-		afree(s, ATEMP);	\
-} while (0)
-#define afreechv(v,s)	do {		\
-	if (v)				\
-		afree(s, ATEMP);	\
-} while (0)
+void afree(void *, Area *);	/* can take NULL */
 /* edit.c */
 void x_init(void);
 int x_read(char *, size_t);
 int x_bind(const char *, const char *, int, int);
 /* UTF-8 hack stuff */
+size_t utf_mbtowc(unsigned int *, const char *);
+size_t utf_wctomb(char *, unsigned int);
+void utf_cptradj(const char *, const char **);
 int utf_widthadj(const char *, const char **);
+int utf_mbswidth(const char *);
+int utf_wcwidth(unsigned int);
+const char *utf_skipcols(const char *, int);
 /* eval.c */
 char *substitute(const char *, int);
 char **eval(const char **, int);
@@ -1282,6 +1296,9 @@ int c_test(const char **);
 #if HAVE_MKNOD
 int c_mknod(const char **);
 #endif
+#if HAVE_REALPATH
+int c_realpath(const char **);
+#endif
 int c_rename(const char **);
 /* histrap.c */
 void init_histvec(void);
@@ -1405,12 +1422,23 @@ struct tbl **ktsort(struct table *);
 /* misc.c */
 void setctypes(const char *, int);
 void initctypes(void);
-#ifdef MKSH_SMALL
+#if defined(MKSH_SMALL) || !HAVE_EXPSTMT
+#define str_save_		str_save
+#define str_nsave_		str_nsave
 char *str_save(const char *, Area *);
-#else
-#define str_save(s,ap) (str_nsave((s), (s) ? strlen(s) : 0, (ap)))
-#endif
 char *str_nsave(const char *, int, Area *);
+#else
+#define str_save_(s,ap)		str_nsave_((s), strlen(s), (ap))
+#define str_nsave_(s,n,ap)	({				\
+	size_t str_save_sz = (n) + 1;				\
+	char *str_save_rv = alloc(str_save_sz, (ap));		\
+	strlcpy(str_save_rv, (s), str_save_sz);			\
+	(str_save_rv);						\
+})
+#define str_save(s,ap)		(!(s) ? NULL : str_save_((s), (ap)))
+#define str_nsave(s,n,ap)	\
+	(!(s) || (n) < 0 ? NULL : str_nsave_((s), (n), (ap)))
+#endif
 size_t option(const char *);
 char *getoptions(void);
 void change_flag(enum sh_flag, int, char);
@@ -1437,7 +1465,7 @@ void simplify_path(char *);
 char *get_phys_path(const char *);
 void set_current_wd(char *);
 #if !HAVE_EXPSTMT
-bool ksh_isspace_(unsigned);
+bool ksh_isspace_(unsigned int);
 #endif
 /* shf.c */
 struct shf *shf_open(const char *, int, int, int);
