@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/syn.c,v 1.20 2008/04/01 22:20:20 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/syn.c,v 1.25 2008/07/18 11:33:13 tg Exp $");
 
 struct nesting_state {
 	int start_token;	/* token than began nesting (eg, FOR) */
@@ -144,10 +144,17 @@ static struct ioword *
 synio(int cf)
 {
 	struct ioword *iop;
+	static struct ioword *nextiop = NULL;
 	int ishere;
 
+	if (nextiop != NULL) {
+		iop = nextiop;
+		nextiop = NULL;
+		return (iop);
+	}
+
 	if (tpeek(cf) != REDIR)
-		return NULL;
+		return (NULL);
 	ACCEPT;
 	iop = yylval.iop;
 	ishere = (iop->flag&IOTYPE) == IOHERE;
@@ -161,7 +168,18 @@ synio(int cf)
 		*herep++ = iop;
 	} else
 		iop->name = yylval.cp;
-	return iop;
+
+	if (iop->flag & IOBASH) {
+		nextiop = (struct ioword *)alloc(sizeof (*iop), ATEMP);
+
+		iop->flag &= ~IOBASH;
+		nextiop->unit = 2;
+		nextiop->flag = IODUP;
+		nextiop->name = shf_smprintf("%d", iop->unit);
+		nextiop->delim = NULL;
+		nextiop->heredoc = NULL;
+	}
+	return (iop);
 }
 
 static struct op *
@@ -186,7 +204,7 @@ get_command(int cf)
 	XPtrV args, vars;
 	struct nesting_state old_nesting;
 
-	iops = (struct ioword **) alloc(sizeofN(struct ioword *, NUFILE+1),
+	iops = (struct ioword **)alloc(sizeofN(struct ioword *, NUFILE+1),
 	    ATEMP);
 	XPinit(args, 16);
 	XPinit(vars, 16);
@@ -211,9 +229,11 @@ get_command(int cf)
 			    (XPsize(args) == 0 ? ALIAS|VARASN : CMDWORD);
 			switch (tpeek(cf)) {
 			case REDIR:
-				if (iopn >= NUFILE)
-					yyerror("too many redirections\n");
-				iops[iopn++] = synio(cf);
+				while ((iop = synio(cf)) != NULL) {
+					if (iopn >= NUFILE)
+						yyerror("too many redirections\n");
+					iops[iopn++] = iop;
+				}
 				break;
 
 			case LWORD:
@@ -346,7 +366,7 @@ get_command(int cf)
 		if (!is_wdvarname(yylval.cp, true))
 			yyerror("%s: bad identifier\n",
 			    c == FOR ? "for" : "select");
-		t->str = str_nsave(ident, strlen(ident), ATEMP);
+		t->str = str_save_(ident, ATEMP);
 		nesting_push(&old_nesting, c);
 		t->vars = wordlist();
 		t->left = dogroup();
@@ -391,6 +411,11 @@ get_command(int cf)
 	case TIME:
 		syniocf &= ~(KEYWORD|ALIAS);
 		t = pipeline(0);
+		if (t) {
+			t->str = alloc(2, ATEMP);
+			t->str[0] = 0;	/* TF_* flags */
+			t->str[1] = '\0';
+		}
 		t = block(TTIME, t, NOBLOCK, NOWORDS);
 		break;
 
@@ -588,13 +613,14 @@ function_body(char *name,
 		 * be used as input), we pretend there is a colon here.
 		 */
 		t->left = newtp(TCOM);
-		t->left->args = (const char **)alloc(sizeof(char *) * 2, ATEMP);
-		t->left->args[0] = tv = alloc(sizeof(char) * 3, ATEMP);
+		t->left->args = (const char **)alloc(sizeof (char *) * 2,
+		    ATEMP);
+		t->left->args[0] = tv = alloc(sizeof (char) * 3, ATEMP);
 		tv[0] = CHAR;
 		tv[1] = ':';
 		tv[2] = EOS;
 		t->left->args[1] = NULL;
-		t->left->vars = (char **)alloc(sizeof(char *), ATEMP);
+		t->left->vars = (char **)alloc(sizeof (char *), ATEMP);
 		t->left->vars[0] = NULL;
 		t->left->lineno = 1;
 	}
@@ -771,7 +797,7 @@ newtp(int type)
 {
 	struct op *t;
 
-	t = (struct op *)alloc(sizeof(*t), ATEMP);
+	t = (struct op *)alloc(sizeof (*t), ATEMP);
 	t->type = type;
 	t->u.evalflags = 0;
 	t->args = NULL;
