@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/tree.c,v 1.15 2008/03/01 21:10:26 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/tree.c,v 1.18 2008/07/09 21:32:45 tg Exp $");
 
 #define INDENT	4
 
@@ -197,7 +197,7 @@ pioact(struct shf *shf, int indent, struct ioword *iop)
 	    (type == IODUP && (iop->unit == !(flag & IORDUP))) ? iop->unit :
 	    iop->unit + 1;
 	if (iop->unit != expected)
-		tputc('0' + iop->unit, shf);
+		shf_fprintf(shf, "%d", iop->unit);
 
 	switch (type) {
 	case IOREAD:
@@ -375,7 +375,7 @@ vfptreef(struct shf *shf, int indent, const char *fmt, va_list va)
 				shf_fprintf(shf, "%d", va_arg(va, int));
 				break;
 			case 'u':	/* decimal */
-				shf_fprintf(shf, "%u", va_arg(va, unsigned));
+				shf_fprintf(shf, "%u", va_arg(va, unsigned int));
 				break;
 			case 'T':	/* format tree */
 				ptree(va_arg(va, struct op *), indent, shf);
@@ -421,7 +421,7 @@ tcopy(struct op *t, Area *ap)
 	if (t == NULL)
 		return NULL;
 
-	r = (struct op *) alloc(sizeof(struct op), ap);
+	r = (struct op *)alloc(sizeof (struct op), ap);
 
 	r->type = t->type;
 	r->u.evalflags = t->u.evalflags;
@@ -433,8 +433,8 @@ tcopy(struct op *t, Area *ap)
 	else {
 		for (tw = (const char **)t->vars; *tw++ != NULL; )
 			;
-		rw = r->vars = (char **)
-		    alloc((tw - (const char **)t->vars + 1) * sizeof(*tw), ap);
+		rw = r->vars = (char **)alloc((tw -
+		    (const char **)t->vars + 1) * sizeof (*tw), ap);
 		for (tw = (const char **)t->vars; *tw != NULL; )
 			*rw++ = wdcopy(*tw++, ap);
 		*rw = NULL;
@@ -445,8 +445,8 @@ tcopy(struct op *t, Area *ap)
 	else {
 		for (tw = t->args; *tw++ != NULL; )
 			;
-		r->args = (const char **)(rw = (char **)
-			alloc((tw - t->args + 1) * sizeof(*tw), ap));
+		r->args = (const char **)(rw = (char **)alloc((tw -
+		    t->args + 1) * sizeof (*tw), ap));
 		for (tw = t->args; *tw != NULL; )
 			*rw++ = wdcopy(*tw++, ap);
 		*rw = NULL;
@@ -615,13 +615,13 @@ iocopy(struct ioword **iow, Area *ap)
 
 	for (ior = iow; *ior++ != NULL; )
 		;
-	ior = (struct ioword **) alloc((ior - iow + 1) * sizeof(*ior), ap);
+	ior = (struct ioword **)alloc((ior - iow + 1) * sizeof (*ior), ap);
 
 	for (i = 0; iow[i] != NULL; i++) {
 		struct ioword *p, *q;
 
 		p = iow[i];
-		q = (struct ioword *) alloc(sizeof(*p), ap);
+		q = (struct ioword *)alloc(sizeof (*p), ap);
 		ior[i] = q;
 		*q = *p;
 		if (p->name != NULL)
