@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/var.c,v 1.51 2008/02/24 15:20:52 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/var.c,v 1.57 2008/07/12 16:56:40 tg Exp $");
 
 /*
  * Variables
@@ -37,7 +37,7 @@ newblock(void)
 	struct block *l;
 	static const char *empty[] = { null };
 
-	l = (struct block *) alloc(sizeof(struct block), ATEMP);
+	l = (struct block *)alloc(sizeof (struct block), ATEMP);
 	l->flags = 0;
 	ainit(&l->area); /* todo: could use e->area (l->area => l->areap) */
 	if (!e->loc) {
@@ -130,7 +130,7 @@ array_index_calc(const char *n, bool *arrayp, uint32_t *valp)
 
 		/* Calculate the value of the subscript */
 		*arrayp = true;
-		tmp = str_nsave(p+1, len-2, ATEMP);
+		tmp = str_nsave(p + 1, len - 2, ATEMP);
 		sub = substitute(tmp, 0);
 		afree(tmp, ATEMP);
 		n = str_nsave(n, p - n, ATEMP);
@@ -298,19 +298,31 @@ str_val(struct tbl *vp)
 			n = (vp->val.i < 0) ? -vp->val.i : vp->val.i;
 		base = (vp->type == 0) ? 10 : vp->type;
 
-		*--s = '\0';
-		do {
-			*--s = digits[n % base];
-			n /= base;
-		} while (n != 0);
-		if (base != 10) {
-			*--s = '#';
-			*--s = digits[base % 10];
-			if (base >= 10)
-				*--s = digits[base / 10];
+		if (base == 1) {
+			size_t sz = 1;
+
+			*(s = strbuf) = '1';
+			s[1] = '#';
+			if (!Flag(FUTFHACK) || ((n & 0xFF80) == 0xEF80))
+				s[2] = n & 0xFF;
+			else
+				sz = utf_wctomb(s + 2, n);
+			s[2 + sz] = '\0';
+		} else {
+			*--s = '\0';
+			do {
+				*--s = digits[n % base];
+				n /= base;
+			} while (n != 0);
+			if (base != 10) {
+				*--s = '#';
+				*--s = digits[base % 10];
+				if (base >= 10)
+					*--s = digits[base / 10];
+			}
+			if (!(vp->flag & INT_U) && vp->val.i < 0)
+				*--s = '-';
 		}
-		if (!(vp->flag & INT_U) && vp->val.i < 0)
-			*--s = '-';
 		if (vp->flag & (RJUST|LJUST)) /* case already dealt with */
 			s = formatstr(vp, s);
 		else
@@ -374,7 +386,7 @@ setstr(struct tbl *vq, const char *s, int error_ok)
 	vq->flag |= ISSET;
 	if ((vq->flag&SPECIAL))
 		setspec(vq);
-	afreechk(salloc);
+	afree(salloc, ATEMP);
 	return (1);
 }
 
@@ -401,9 +413,8 @@ int
 getint(struct tbl *vp, long int *nump, bool arith)
 {
 	char *s;
-	int c;
-	int base, neg;
-	int have_base = 0;
+	int c, base, neg;
+	bool have_base = false;
 	long num;
 
 	if (vp->flag&SPECIAL)
@@ -431,18 +442,28 @@ getint(struct tbl *vp, long int *nump, bool arith)
 				s++;
 		} else
 			base = 8;
-		have_base++;
+		have_base = true;
 	}
 	for (c = *s++; c ; c = *s++) {
 		if (c == '-') {
 			neg++;
 			continue;
 		} else if (c == '#') {
-			base = (int) num;
-			if (have_base || base < 2 || base > 36)
-				return -1;
+			base = (int)num;
+			if (have_base || base < 1 || base > 36)
+				return (-1);
+			if (base == 1) {
+				unsigned int wc;
+
+				if (!Flag(FUTFHACK))
+					wc = *(unsigned char *)s;
+				else if (utf_mbtowc(&wc, s) == (size_t)-1)
+					wc = 0xEF00 + *(unsigned char *)s;
+				*nump = (long)wc;
+				return (1);
+			}
 			num = 0;
-			have_base = 1;
+			have_base = true;
 			continue;
 		} else if (ksh_isdigit(c))
 			c -= '0';
@@ -491,8 +512,9 @@ formatstr(struct tbl *vp, const char *s)
 {
 	int olen, nlen;
 	char *p, *q;
+	size_t psiz;
 
-	olen = strlen(s);
+	olen = utf_mbswidth(s);
 
 	if (vp->flag & (RJUST|LJUST)) {
 		if (!vp->u2.field)	/* default field width */
@@ -501,25 +523,41 @@ formatstr(struct tbl *vp, const char *s)
 	} else
 		nlen = olen;
 
-	p = (char *) alloc(nlen + 1, ATEMP);
+	p = (char *)alloc((psiz = nlen * /* MB_LEN_MAX */ 3 + 1), ATEMP);
 	if (vp->flag & (RJUST|LJUST)) {
-		int slen;
+		int slen = olen, i = 0;
 
 		if (vp->flag & RJUST) {
-			const char *qq = s + olen;
+			const char *qq = s;
+			int n = 0;
+
+			while (i < slen)
+				i += utf_widthadj(qq, &qq);
 			/* strip trailing spaces (at&t uses qq[-1] == ' ') */
-			while (qq > s && ksh_isspace(qq[-1]))
+			while (qq > s && ksh_isspace(qq[-1])) {
 				--qq;
-			slen = qq - s;
-			if (slen > vp->u2.field) {
-				s += slen - vp->u2.field;
-				slen = vp->u2.field;
+				--slen;
 			}
+			if (vp->flag & ZEROFIL && vp->flag & INTEGER) {
+				if (s[1] == '#')
+					n = 2;
+				else if (s[2] == '#')
+					n = 3;
+				if (vp->u2.field <= n)
+					n = 0;
+			}
+			if (n) {
+				memcpy(p, s, n);
+				s += n;
+			}
+			while (slen > vp->u2.field)
+				slen -= utf_widthadj(s, &s);
 			if (vp->u2.field - slen)
-				memset(p, (vp->flag & ZEROFIL) ? '0' : ' ',
+				memset(p + n, (vp->flag & ZEROFIL) ? '0' : ' ',
 				    vp->u2.field - slen);
+			slen -= n;
 			shf_snprintf(p + vp->u2.field - slen,
-			    nlen + 1 - (vp->u2.field - slen),
+			    psiz - (vp->u2.field - slen),
 			    "%.*s", slen, s);
 		} else {
 			/* strip leading spaces/zeros */
@@ -532,7 +570,7 @@ formatstr(struct tbl *vp, const char *s)
 				vp->u2.field, vp->u2.field, s);
 		}
 	} else
-		memcpy(p, s, olen + 1);
+		memcpy(p, s, strlen(s) + 1);
 
 	if (vp->flag & UCASEV_AL) {
 		for (q = p; *q; q++)
@@ -604,9 +642,10 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 		}
 		val += len;
 	}
-	if (*val == '=')
-		tvar = str_nsave(var, val++ - var, ATEMP);
-	else {
+	if (*val == '=') {
+		int i = val++ - var;
+		tvar = str_nsave(var, i, ATEMP);
+	} else {
 		/* Importing from original environment: must have an = */
 		if (set & IMPORT)
 			return NULL;
@@ -1161,7 +1200,7 @@ arraysearch(struct tbl *vp, uint32_t val)
 		else
 			new = curr;
 	} else
-		new = (struct tbl *)alloc(sizeof(struct tbl) + namelen,
+		new = (struct tbl *)alloc(sizeof (struct tbl) + namelen,
 		    vp->areap);
 	strlcpy(new->name, vp->name, namelen);
 	new->flag = vp->flag & ~(ALLOC|DEFINED|ISSET|SPECIAL);
